from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ItemRevision, BOMWindow
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FileTicketDetails(TcBaseObj):
    """
    File ticket details.
    
    :var filename: The name of the log file.
    :var fileTicket: The file management server ticket of the file.
    """
    filename: str = ''
    fileTicket: str = ''


@dataclass
class GetClusterDetails(TcBaseObj):
    """
    in
    
    :var clusters: GetClusterDetails structure contains  the elements that are obtained initially by calling the
    operation searchForClusters. These elements contain BOMLine objects for which properties need to be obtained.
    """
    clusters: List[SearchForClustersElement] = ()


@dataclass
class SearchForClusters(TcBaseObj):
    """
    The input element specifying the parameters to be used for searching for patterns.
    
    :var mbomModelAssemblyNode: The mbom assembly node that is to be used a template for finding the candidates forming
    the set for pattern matching in ebom. Currently, must be a BOMLine object.
    :var ebomContext: The ebom context below which to search for patterns. Currently, the context must be a BOMLine
    object.
    """
    mbomModelAssemblyNode: BusinessObject = None
    ebomContext: BusinessObject = None


@dataclass
class SearchForClustersElement(TcBaseObj):
    """
    SearchForClustersElement structure contains the BOMLine object, its ID in Context (Top Level)  and whether it is a
    basis for formulating a pattern.
    
    :var line: The BOMLine.
    :var isBasis: If true, indicates that it was an element used as a basis to compute the rest of the transforms in
    the pattern list.
    :var idic: The ID in Context (Top Level) of the BOMLine.
    """
    line: BusinessObject = None
    isBasis: bool = False
    idic: str = ''


@dataclass
class SearchForClustersResponse(TcBaseObj):
    """
    SearchForClustersResponse structure contains the response elements (patterns) that match the model assembly in
    MBOM, basis occurrence chain - which is the chain of occurrence threads for the model assembly BOMLine object used
    as a basis for computing the relative transforms, the nodes assigned to the model assembly in MBOM and potentially
    unassigned nodes. The response will also return ServiceData.
    
    :var basisOccurrenceChain: The chain of occurrence threads for the model assembly   BOMLine object used as a basis
    for computing relative transforms. This is computed by concatenating a chain of clone stable ID property value from
    the model assembly down to the basis line (the basis line is the BOMLine object which occurs the least number of
    times in the model assembly in MBOM among the BOMLine objects assigned from the EBOM).
    :var nodesAssignedInModelAssembly: The nodes represented by BOMLine objects assigned in  the model assembly in the
    MBOM.
    :var patterns: A list of patterns. Each pattern is represented by SearchForResponseClusterElement. Each such
    element contains nodes that form the pattern. Each node is of type SearchForClusterElement.
    :var unassignedModelAssemblyNodes: The potentially unassigned model assembly nodes (BOMLine objects) in the model
    assembly.
    :var serviceData: Service data capturing partial errors using the input array index as clientId. ServiceData for
    this operation will not have any data related to properties of BOMLine objects returned, because it is not
    necessary to return properties in this operation, they could be returned from another operation getClusterDetails.
    Also it will be expensive to return properties in this call itself.
    """
    basisOccurrenceChain: str = ''
    nodesAssignedInModelAssembly: List[BusinessObject] = ()
    patterns: List[SearchForClustersResponseElement] = ()
    unassignedModelAssemblyNodes: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class SearchForClustersResponseElement(TcBaseObj):
    """
    SearchForClustersResponseElement structure contains the collection of objects of type SearchForClustersElement.
    
    :var patternNodes: A list of pattern nodes. Each pattern node is represented by struct SearchForClustersElement and
    it comprises of the BOMLine object, the ID in Context (Top Level) of the BOMLine object and the Find number of the
    BOMLine object.
    :var owningModelAssemblies: A list of owning model assemblies ( currently,  BOMLine objects ) below which the
    pattern nodes appear. The same pattern nodes may appear under multiple model assemblies.
    """
    patternNodes: List[SearchForClustersElement] = ()
    owningModelAssemblies: List[BusinessObject] = ()


@dataclass
class ConfigurationsInfo(TcBaseObj):
    """
    Configuration Info Object
    
    :var window: The BOMWindow of the structure.
    :var rootLine: The root BOMLine or BOPLine of the structure. The type of BOPLine object can be Mfg0BvrProcess.
    :var saveCC: The CollaborationContext object being saved, if NULL it will not be saved.
    :var revisionRule: The RevisionRule object.
    :var variantRule: The BOMVariantRule object.
    :var effGrpRevisions: List of Fnd0EffectvtyGrpRevision objects to be applied to the window.
    :var showUnconfigVariants: Show or hide the BOMLine or BOPLine objects configured by the variant configuration on
    the BOMWindow. If true, the configured-out lines are visible; otherwise those are hidden.
    :var showUnconfigOccEffectivity: Show or hide the BOMLine or BOPLine objects configured-out by the occurrence
    effectivity configuration on the BOMWindow. If true, the configured-out lines are visible; otherwise those are
    hidden.
    :var showSupressedOccurrences: Show or hide the suppressed BOMLine or BOPLine objects in the BOMWindow. If true,
    the suppressed lines are visible; otherwise those are hidden.
    :var showUnconfigICs: Show or hide the BOMLine or BOPLine objects configured-out by the configuration of the
    incremental change(IC) context on the BOMWindow. If the value is true, the configured-out lines are visible;
    otherwise those are hidden
    :var showUnconfigAssignedOccs: Show or hide the BOMLine or BOPLine objects which are assigned occurrences from the
    associated BOMLine structure. If true, the assigned lines are visible; otherwise those are hidden.
    """
    window: BOMWindow = None
    rootLine: BusinessObject = None
    saveCC: BusinessObject = None
    revisionRule: BusinessObject = None
    variantRule: BusinessObject = None
    effGrpRevisions: List[ItemRevision] = ()
    showUnconfigVariants: bool = False
    showUnconfigOccEffectivity: bool = False
    showSupressedOccurrences: bool = False
    showUnconfigICs: bool = False
    showUnconfigAssignedOccs: bool = False


@dataclass
class ConfigureMultipleStructuresResponse(TcBaseObj):
    """
    ConfigureMultipleStructuresResponse object.
    
    :var serviceData: Standard service data.
    :var logFileTicket: The details about the file management server ticket of the log file.
    """
    serviceData: ServiceData = None
    logFileTicket: FileTicketDetails = None


@dataclass
class CreateReuseAssembly(TcBaseObj):
    """
    CreateReuseAssembly structure contains the input for creating new assemblies under given parent BOMLine objects.
    
    :var parentNodes: The  BOMLine objects in the MBOM under which new assemblies matching the model assembly will be
    created.
    :var modelAssemblyNode: The BOMLine representing the model assembly in MBOM.
    :var basisOccurrenceChain: The occurrence chain obtained from a prior call to searchForClusters representing the
    chain for the node used as a basis for computing relative transforms.
    :var nodesAssignedInModelAssembly: The currently assigned BOMLine objects under model assembly in the MBOM. These
    are obtained from the response of a prior call to searchFor operation.
    :var clusterElements: The cluster elements represented by SearchForClustersElement returned by prior call to
    searchForClusters operation.
    """
    parentNodes: List[BusinessObject] = ()
    modelAssemblyNode: BusinessObject = None
    basisOccurrenceChain: str = ''
    nodesAssignedInModelAssembly: List[BusinessObject] = ()
    clusterElements: List[SearchForClustersElement] = ()


@dataclass
class CreateReuseAssemblyResp(TcBaseObj):
    """
    A list of created nodes ( currently, BOMLine objects ), one per input parentNode. Can be NULL if there is a failure.
    
    :var nodes: A list of created nodes ( currently, BOMLine objects ), one per input parentNode.
    :var serviceData: serviceData
    """
    nodes: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class AlignAssemblyData(TcBaseObj):
    """
    An element representing the data to be used for aligning a node ( Currently, a BOMLine object ) in source assembly
    to a node in target assembly.
    
    :var sourceNode: The  BOMLine object in the source structure which needs to be aligned with a BOMLine object in the
    target structure.
    :var targetNode: The BOMLine object in the target structure with which the BOMLine object in the source structure
    needs to be aligned.
    :var reuseAssemblyNode: The BOMLine object under which the target BOMLine object may have been assigned. All
    BOMLine objects assigned from source structure under this reuseAssemblyNode will be considered for alignment if
    alignMode is not specified.
    :var alignMode: The alignMode can take one of the values from :
    a)    "AlignIdInTopLevelContextOnly" - used to align the ID in Context (Top Level ) only.
    b)    "AlignOccurrenceProperties" - used to align all occurrence proerpties and ID in Context (Top Level) based on
    the preference "MEAlignedPropertiesList".
    c)    "AlignTransformOnReuseAssemblyOnly" - used to adjust the transform on the parent and stamp ID in Context(Top
    Level) on the source and target BOMLine objects. This is the case where ID in Context (Top Level) is there but they
    need to be paired and the Absolute Transformation Matrix property needs to be adjusted on the model assembly.
    c) "AlignAllModelAssemblies" - used to indicate that on aligning one model assembly, search for others with the
    same item id and align those also.
    d)    An empty value - In this case the pattern defined by the reuseAsemblyNode is searched for in the source
    structure and a new BOMLine object is created that matches the reuseAssemblyNode. Then  all occurrrence properties
    of all BOMLine objects under the new BOMLine object will be adjusted.
    """
    sourceNode: BusinessObject = None
    targetNode: BusinessObject = None
    reuseAssemblyNode: BusinessObject = None
    alignMode: str = ''



from __future__ import annotations

from tcsoa.gen.AuthorizedDataAccess._2018_06.LicenseManagement import LicenseDetails
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class LicenseManagementService(TcService):

    @classmethod
    def createOrUpdateLicense(cls, licenseInfo: List[LicenseDetails]) -> ServiceData:
        """
        This operation creates or modifies an ADA_License business object for each LicenseDetails supplied. The
        LicenseDetails contains information for properties such as license type, license ID, category, citizenship,
        expiry date, lock date, reason, in accordance with, and associated users and groups, for a given license. If a
        specified license ID already exists, the rest of the property values are updated on that license. However, if
        the license ID does not exist, then a new license of the specified type and ID will be created, and the rest of
        the properties are set on the created license. The user performing the operation will need the privilege
        specified in the ADA_license_administration_privilege site preference to create or modify an ADA License. If a
        user does not have the necessary privilege or if there is a validation error, the operation would fail and the
        error is returned in the ServiceData.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateLicense',
            library='AuthorizedDataAccess',
            service_date='2018_06',
            service_name='LicenseManagement',
            params={'licenseInfo': licenseInfo},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Core._2006_03.DataManagement import AttributeNameValueMap, GenerateRevisionIdsProperties, GenerateItemIdsAndInitialRevisionIdsResponse, ReviseResponse, ItemProperties, ItemRevPropertyMap, CreateRelationsResponse, DatasetProperties, CreateDatasetsResponse, GenerateItemIdsAndInitialRevisionIdsProperties, CreateFoldersResponse, Relationship, CreateFolderInput, CreateItemsResponse, GenerateRevisionIdsResponse, ObjectOwner
from tcsoa.gen.BusinessObjects import BusinessObject, GroupMember, ImanFile
from tcsoa.gen.Core._2006_03.FileManagement import CommitDatasetFileInfo, GetDatasetWriteTicketsResponse, GetDatasetWriteTicketsInputData, FileTicketsResponse
from tcsoa.gen.Core._2006_03.Session import GetGroupMembershipResponse, GetSessionGroupMemberResponse, PreferencesResponse, GetAvailableServicesResponse, LoginResponse, PrefSetting
from tcsoa.gen.Core._2006_03.Reservation import GetReservationHistoryResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def getProperties(cls, objects: List[BusinessObject], attributes: List[str]) -> ServiceData:
        """
        This service operation is provided to get property values of instances outside of the current object property
        policy for a particular business object.  Net result of this operation includes the properties expressly
        defined in the input attributes as well as the properties defined in the current Object Property Policy.
        
        This operation takes care of following:
        - Since all relations are stored as properties on the primary object, this operation supports the expanding of
        relations.  
        - Properties in the input attribute argument that reference other objects  (relations) will have the properties
        for those referenced objects returned as defined by the Object Property Policy.
        
        
        Limitation:
        - Classification objects attached to WorkspaceObjects using "IMAN_classification" relation are not returned by
        this operation. User need to use findClassificationObjects operation from Classification service to retrieve
        properties of such objects. For more information about findClassificationObjects operation please refer
        classification service guide.
        
        """
        return cls.execute_soa_method(
            method_name='getProperties',
            library='Core',
            service_date='2006_03',
            service_name='DataManagement',
            params={'objects': objects, 'attributes': attributes},
            response_cls=ServiceData,
        )

    @classmethod
    def revise(cls, input: ItemRevPropertyMap) -> ReviseResponse:
        """
        Revises a list of next Item Revisions based on input existing Item Revision object references and any
        additional attributes.  Uses deep copy rules to propagate existing relations from the Item Revision or to
        create new references.
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='revise',
            library='Core',
            service_date='2006_03',
            service_name='DataManagement',
            params={'input': input},
            response_cls=ReviseResponse,
        )

    @classmethod
    def setDisplayProperties(cls, objects: List[BusinessObject], attributes: AttributeNameValueMap) -> ServiceData:
        """
        This operation is provided to update the Teamcenter objects for the given name/display value pairs. This
        operation works for all supported property value types to set display values. When setting this operation it
        invokes the server PROP_UIF_set_value extensions. For updating an array property, display values need to be
        comma (,) separated which server parses them into individual values before assigning.
        Note:  Since LOVs support the display as feature where internal values of the LOV can be different from
        displayable values, this operation expects that internal value of the selection to be passed and not the
        display value.
        """
        return cls.execute_soa_method(
            method_name='setDisplayProperties',
            library='Core',
            service_date='2006_03',
            service_name='DataManagement',
            params={'objects': objects, 'attributes': attributes},
            response_cls=ServiceData,
        )

    @classmethod
    def createDatasets(cls, input: List[DatasetProperties]) -> CreateDatasetsResponse:
        """
        This operation creates a list of Datasets and creates the specified relation between created Dataset and input
        container object.
        """
        return cls.execute_soa_method(
            method_name='createDatasets',
            library='Core',
            service_date='2006_03',
            service_name='DataManagement',
            params={'input': input},
            response_cls=CreateDatasetsResponse,
        )

    @classmethod
    def createFolders(cls, folders: List[CreateFolderInput], container: BusinessObject, relationType: str) -> CreateFoldersResponse:
        """
        This operation creates a list of new Folder objects with the given names, descriptions and attaches them to the
        parent container. If attaching a created Folder to its parent container fails, the Folder will not be deleted.
        """
        return cls.execute_soa_method(
            method_name='createFolders',
            library='Core',
            service_date='2006_03',
            service_name='DataManagement',
            params={'folders': folders, 'container': container, 'relationType': relationType},
            response_cls=CreateFoldersResponse,
        )

    @classmethod
    def createItems(cls, properties: List[ItemProperties], container: BusinessObject, relationType: str) -> CreateItemsResponse:
        """
        This operation creates a list of Items and associated data (ItemRevision/ItemMasterForm/ItemRevisionMasterForm)
        based on the input attribute structures and the specified relation type between created Item and input object. 
        If container and relation type are not input, none of the Items will be related to a container. (There is no
        default, if any destination is desired, it must be provided as input). Note: createItems are for items
        creation, if a compound object such as ItemRevision adds a required property in BMIDE, createItems will fail
        since it only accepts required properties for item types, not for its associated data such as ItemRevision.
        Also, if any other properties including object description and custom properties are added as required on Item,
        createItems will fail. In this case, user should use 'createObjects' instead.
        """
        return cls.execute_soa_method(
            method_name='createItems',
            library='Core',
            service_date='2006_03',
            service_name='DataManagement',
            params={'properties': properties, 'container': container, 'relationType': relationType},
            response_cls=CreateItemsResponse,
        )

    @classmethod
    def createRelations(cls, input: List[Relationship]) -> CreateRelationsResponse:
        """
        Creates the specified relation between the input objects (primary and secondary objects). If the relation name
        is not specified then default relation name specified in either the preference
        ParentTypeName_ChildTypeName_default_relation or ParentTypeName_default_relation is considered as the relation
        name. In case none of these preferences are set the relation between the primary and secondary object is not
        created. If the primary object has a relation property by the specified relation name, then the secondary
        object is associated with the primary object through the relation property.
        """
        return cls.execute_soa_method(
            method_name='createRelations',
            library='Core',
            service_date='2006_03',
            service_name='DataManagement',
            params={'input': input},
            response_cls=CreateRelationsResponse,
        )

    @classmethod
    def deleteObjects(cls, objects: List[BusinessObject]) -> ServiceData:
        """
        This operation deletes the input objects.  In the case of Item, it also deletes all ItemRevision objects, 
        associated ItemMaster, ItemRevisionMaster forms, and Dataset objects.  The input object can be an ImanRelation.
        """
        return cls.execute_soa_method(
            method_name='deleteObjects',
            library='Core',
            service_date='2006_03',
            service_name='DataManagement',
            params={'objects': objects},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteRelations(cls, input: List[Relationship]) -> ServiceData:
        """
        Deletes the specified relation between the primary and secondary object for each input structure.
        """
        return cls.execute_soa_method(
            method_name='deleteRelations',
            library='Core',
            service_date='2006_03',
            service_name='DataManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def generateItemIdsAndInitialRevisionIds(cls, input: List[GenerateItemIdsAndInitialRevisionIdsProperties]) -> GenerateItemIdsAndInitialRevisionIdsResponse:
        """
        This operation generates a list of Item IDs and initial ItemRevision IDs.  The initial revision ID is defined
        as the first revision ID for the given type.
        """
        return cls.execute_soa_method(
            method_name='generateItemIdsAndInitialRevisionIds',
            library='Core',
            service_date='2006_03',
            service_name='DataManagement',
            params={'input': input},
            response_cls=GenerateItemIdsAndInitialRevisionIdsResponse,
        )

    @classmethod
    def generateRevisionIds(cls, input: List[GenerateRevisionIdsProperties]) -> GenerateRevisionIdsResponse:
        """
        This operation generates a set of revision IDs.  The ID can be either the next ID for an existing Item or the
        first revision ID for a new Item.
        """
        return cls.execute_soa_method(
            method_name='generateRevisionIds',
            library='Core',
            service_date='2006_03',
            service_name='DataManagement',
            params={'input': input},
            response_cls=GenerateRevisionIdsResponse,
        )

    @classmethod
    def changeOwnership(cls, input: List[ObjectOwner]) -> ServiceData:
        """
        This operation can be used to change the ownership on a given business object to the given user and group. 
        Owning user attribute on the business object will be changed to the given user and owning group attribute is
        changed to the given group.  The user needs CHANGE_OWNER privilege and WRITE privilege on the business object
        to be able to change its ownership.  If user does not have the required privileges then this operation will
        return error code 515001. If the given user is invalid or given group is invalid then this operation will
        return error code 515024.
        
        Use cases:
        Change owner menu action calls this operation.
        """
        return cls.execute_soa_method(
            method_name='changeOwnership',
            library='Core',
            service_date='2006_03',
            service_name='DataManagement',
            params={'input': input},
            response_cls=ServiceData,
        )


class ReservationService(TcService):

    @classmethod
    def getReservationHistory(cls, objects: List[BusinessObject]) -> GetReservationHistoryResponse:
        """
        This operation gets the reservation history for a set of business objects, such as Dataset, Item, ItemRevision
        and many other business object types.  An object which has never been checked out will have a valid reservation
        history with an empty sequence of events.
        """
        return cls.execute_soa_method(
            method_name='getReservationHistory',
            library='Core',
            service_date='2006_03',
            service_name='Reservation',
            params={'objects': objects},
            response_cls=GetReservationHistoryResponse,
        )

    @classmethod
    def checkin(cls, objects: List[BusinessObject]) -> ServiceData:
        """
        This operation checks-in a set of previously checked-out business objects. This operation takes care of all
        complex business logic involved to check-in passed in business objects.  Each input object is verified that it
        is locally owned, site owned, and not transferred to another user after the checkout was performed. This
        operation validates precondition defined per type in COTS object and site customization and performs basic
        check-in. Dataset, ItemRevision and many other business object types have their own business logic for
        check-in. This operation calls underlying 'checkin' method of those individual objects.
        """
        return cls.execute_soa_method(
            method_name='checkin',
            library='Core',
            service_date='2006_03',
            service_name='Reservation',
            params={'objects': objects},
            response_cls=ServiceData,
        )

    @classmethod
    def checkout(cls, objects: List[BusinessObject], comment: str, changeId: str) -> ServiceData:
        """
        This operation checks  out a set of business objects with given comment and change identifier. Only one user
        can perform a check-out transaction on the object. The user must have sufficient  privilege on the object or
        the check out will fail. This operation allows for remote check-out and records the check-out transaction event.
        """
        return cls.execute_soa_method(
            method_name='checkout',
            library='Core',
            service_date='2006_03',
            service_name='Reservation',
            params={'objects': objects, 'comment': comment, 'changeId': changeId},
            response_cls=ServiceData,
        )

    @classmethod
    def cancelCheckout(cls, objects: List[BusinessObject]) -> ServiceData:
        """
        This operation cancels a check-out for a set of previously checked-out business objects. The objects will be
        restored to the pre-check-out state. Only one user can perform a cancel check-out transaction on the object if
        the user has enough privilege on the object.  This action may be applied to remote checked out objects, and
        will cancel the check-out and records the cancel check-out transaction event. Cancel checkout is not supported
        for some of the business objects for e.g. - Item, BOMView,BOMViewRevision, Schedule.
        """
        return cls.execute_soa_method(
            method_name='cancelCheckout',
            library='Core',
            service_date='2006_03',
            service_name='Reservation',
            params={'objects': objects},
            response_cls=ServiceData,
        )


class SessionService(TcService):

    @classmethod
    def getSessionGroupMember(cls) -> GetSessionGroupMemberResponse:
        """
        Get the Group and Role for the current session. The group and role are set at login, either to default values
        or as specified by the input arguments to the login operation. The group and role can be changed at any time
        throughout the session through the 'setSessionGroupMember' or 'setUserSessionState' operations.
        """
        return cls.execute_soa_method(
            method_name='getSessionGroupMember',
            library='Core',
            service_date='2006_03',
            service_name='Session',
            params={},
            response_cls=GetSessionGroupMemberResponse,
        )

    @classmethod
    def login(cls, username: str, password: str, group: str, role: str, sessionDiscriminator: str) -> LoginResponse:
        """
        Authenticates the user's credentials and initialize a Teamcenter session for this client. The operation will
        throw an 'InvalidCredentialsException' if the 'username', 'password' or 'group' is not valid.
        When the client application is deployed to a 4Tier environment (communication through HTTP or TCCS) the login
        operation also contributes to the assignment of a Teamcenter server instance to the client session. The
        Teamcenter architecture varies from other client server architectures in that there is a dedicated instance of
        the Teamcenter server per client application. However, there are use cases where it is desirable for a single
        user to have multiple desktop applications running and each sharing a single instance of a Teamcenter server.
        This is controlled through the following elements:
        - 'hostPath'    From the Connection class constructor, this specifies  the address (URI) the Teamcenter server
        is hosted on.
        - 'username'    From this login operation, this specifies the user's Teamcenter user name.
        - 'sessionDiscriminator'    From this login operation, this identifies the client session.
        
        
        
        The 'hostPath' argument determines the server machine that the client connects to. Once there, the pool manager
        on that host uses the 'username' and 'sessionDiscriminator' arguments of the logon request to determine which
        Teamcenter server instance to assign the client to. If the pool manager has an existing Teamcenter server
        instance with the 'username'/'sessionDiscriminator' key, the client is assigned to that existing instance of
        the Teamcenter server, and therefore sharing the server with another client; otherwise, a new instance of the
        Teamcenter server is used. There are a few general scenarios for the 'sessionDiscriminator' argument:
        
        - Blank     If the user jdoe logs on to Teamcenter using two or more client applications using a blank
        'sessionDiscriminator' argument (for example, jdoe/ ), all of those clients are assigned to the same Teamcenter
        server instance. These client applications can be running on the same or different client hosts.
        - Constant     If the user jdoe logs on to Teamcenter using two or more client applications using a constant or
        fixed 'sessionDiscriminator' argument (for example, jdoe/MyApp ), those clients are assigned to the same
        Teamcenter server instance. This is similar to the blank 'sessionDiscriminator' argument; the difference is
        that only multiple instances of the client application using myApp started by jdoe share the same Teamcenter
        server instance.
        - Unique     If the user jdoe logs on using a unique random-generated string (for example, jdoe/akdk938lakc),
        the client application will be assigned to a dedicated instance of the Teamcenter server.
        
        
         
        The scenario you use depends on how your client application is used in the integrated environment. The most
        common case is the unique 'sessionDiscriminator' value.
        
        Exceptions:
        >When the credentials supplied are invalid or the requested locale is not allowed.  
        - 515143:    The logon was refused due to invalid 'username' or 'password'
        - 515144:    The logon was refused due to invalid 'username' or 'password'
        - 515142:    The logon was refused due to an invalid 'group'.
        
        """
        return cls.execute_soa_method(
            method_name='login',
            library='Core',
            service_date='2006_03',
            service_name='Session',
            params={'username': username, 'password': password, 'group': group, 'role': role, 'sessionDiscriminator': sessionDiscriminator},
            response_cls=LoginResponse,
        )

    @classmethod
    def loginSSO(cls, username: str, ssoCredentials: str, group: str, role: str, sessionDiscriminator: str) -> LoginResponse:
        """
        Authenticates the user using Single-Sign-On (SSO) credentials and initialize a Teamcenter session for this
        client. The 'username' and 'ssoCredentials' arguments must be obtained from Teamcenter's Security Services. The
        'SsoCredentials' class offers a simple API to get these values. The operation will throw an
        'InvalidCredentialsException' if the 'username', 'ssoCredentials' or group is not valid.
        When the client application is deployed to a 4Tier environment (communication through HTTP or TCCS) the login
        operation also contributes to the assignment of a Teamcenter server instance to the client session. The
        Teamcenter architecture varies from other client server architectures in that there is a dedicated instance of
        the Teamcenter server per client application. However, there are use cases where it is desirable for a single
        user to have multiple desktop applications running and each sharing a single instance of a Teamcenter server.
        This is controlled through the following elements:
        - 'hostPath'    From the Connection class constructor, this specifies  the address (URI) the Teamcenter server
        is hosted on.
        - 'username'    From this login operation, this specifies the user's Teamcenter user name.
        - 'sessionDiscriminator'    From this login operation, this identifies the client session.
        
        
        
        The 'hostPath' argument determines the server machine that the client connects to. Once there, the pool manager
        on that host uses the 'username' and 'sessionDiscriminator' arguments of the logon request to determine which
        Teamcenter server instance to assign the client to. If the pool manager has an existing Teamcenter server
        instance with the 'username'/'sessionDiscriminator' key, the client is assigned to that existing instance of
        the Teamcenter server, and therefore sharing the server with another client; otherwise, a new instance of the
        Teamcenter server is used. There are a few general scenarios for the sessionDiscriminator argument:
        
        - Blank     If the user jdoe logs on to Teamcenter using two or more client applications using a blank
        'sessionDiscriminator' argument (for example, jdoe/ ), all of those clients are assigned to the same Teamcenter
        server instance. These client applications can be running on the same or different client hosts.
        - Constant     If the user jdoe logs on to Teamcenter using two or more client applications using a constant or
        fixed 'sessionDiscriminator' argument (for example, jdoe/MyApp ), those clients are assigned to the same
        Teamcenter 'server' instance. This is similar to the blank sessionDiscriminator argument; the difference is
        that only multiple instances of the client application using myApp started by jdoe share the same Teamcenter
        server instance.
        - Unique     If the user jdoe logs on using a unique random-generated string (for example, jdoe/akdk938lakc),
        the client application will be assigned to a dedicated instance of the Teamcenter server.
        
        
         
        The scenario you use depends on how your client application is used in the integrated environment. The most
        common case is the unique 'sessionDiscriminator' value.
        
        Exceptions:
        >When the credentials supplied are invalid or the requested locale is not allowed.  
        - 515143:    The logon was refused due to invalid 'username' or 'ssoCredentials'
        - 515144:    The logon was refused due to invalid 'username' or 'ssoCredentials'
        - 515142:    The logon was refused due to an invalid 'group'.
        
        """
        return cls.execute_soa_method(
            method_name='loginSSO',
            library='Core',
            service_date='2006_03',
            service_name='Session',
            params={'username': username, 'ssoCredentials': ssoCredentials, 'group': group, 'role': role, 'sessionDiscriminator': sessionDiscriminator},
            response_cls=LoginResponse,
        )

    @classmethod
    def logout(cls) -> ServiceData:
        """
        Logout and terminate the Teamcenter session. If the Teamcenter server is being shared with multiple client
        applications, it will not be terminated until each client has issued the 'logout'.
        """
        return cls.execute_soa_method(
            method_name='logout',
            library='Core',
            service_date='2006_03',
            service_name='Session',
            params={},
            response_cls=ServiceData,
        )

    @classmethod
    def setPreferences(cls, settings: List[PrefSetting]) -> PreferencesResponse:
        """
        Set preference values
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='setPreferences',
            library='Core',
            service_date='2006_03',
            service_name='Session',
            params={'settings': settings},
            response_cls=PreferencesResponse,
        )

    @classmethod
    def setSessionGroupMember(cls, groupMember: GroupMember) -> ServiceData:
        """
        Set the Group and Role for the current session. The group and role are set at login, either to default values
        or as specified by the input arguments to the login operation. The group and role can be changed at any time
        throughout the session through this operation or the 'setUserSessionState' operations. The
        'getSessionGroupMember' will return the current group and roll.
        """
        return cls.execute_soa_method(
            method_name='setSessionGroupMember',
            library='Core',
            service_date='2006_03',
            service_name='Session',
            params={'groupMember': groupMember},
            response_cls=ServiceData,
        )

    @classmethod
    def getAvailableServices(cls) -> GetAvailableServicesResponse:
        """
        This operation returns a list of services and service operations that this Teamcenter server instance supports.
        This is useful for client applications that expose different functionality based on the version of the
        Teamcenter server it is connecting to.
        """
        return cls.execute_soa_method(
            method_name='getAvailableServices',
            library='Core',
            service_date='2006_03',
            service_name='Session',
            params={},
            response_cls=GetAvailableServicesResponse,
        )

    @classmethod
    def getGroupMembership(cls) -> GetGroupMembershipResponse:
        """
        Get the valid groups and roles for the current user.
        """
        return cls.execute_soa_method(
            method_name='getGroupMembership',
            library='Core',
            service_date='2006_03',
            service_name='Session',
            params={},
            response_cls=GetGroupMembershipResponse,
        )

    @classmethod
    def getPreferences(cls, prefScope: str, prefNames: List[str]) -> PreferencesResponse:
        """
        Get preference values
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='getPreferences',
            library='Core',
            service_date='2006_03',
            service_name='Session',
            params={'prefScope': prefScope, 'prefNames': prefNames},
            response_cls=PreferencesResponse,
        )


class FileManagementService(TcService):

    @classmethod
    def commitDatasetFiles(cls, commitInput: List[CommitDatasetFileInfo]) -> ServiceData:
        """
        This operation supports the upload (addition) of files to a Teamcenter volume.  The mechanism for a client
        application adding files to a Teamcenter volume contains several steps.  This mechanism is implemented in the
        'com.teamcenter.soa.client.FileManagementUtility' class, which provides this functionality to clients in a
        consistent, reusable package.    The 'com.teamcenter.soa.client.FileManagementUtility' class invokes this
        operation after successfully uploading a file to a Teamcenter volume.
        This operation was unintentionally published.  It is supported only for internal Siemens PLM purposes. 
        Customers should not invoke this operation.
        
        Use cases:
        This operation supports the upload (addition) of files representing named references of a Dataset object to a
        Teamcenter volume.
        """
        return cls.execute_soa_method(
            method_name='commitDatasetFiles',
            library='Core',
            service_date='2006_03',
            service_name='FileManagement',
            params={'commitInput': commitInput},
            response_cls=ServiceData,
        )

    @classmethod
    def getDatasetWriteTickets(cls, inputs: List[GetDatasetWriteTicketsInputData]) -> GetDatasetWriteTicketsResponse:
        """
        This operation obtains File Management System (FMS) write tickets and file storage information for a set of
        supplied Dataset objects. The write tickets are used to transfer files from a local storage to Teamcenter
        volume, and the file storage information can be used to commit Dataset objects referencing those transferred
        files.
        The caller will provide a vector of 'GetDatasetWriteTicketsInputData' objects that contains one or more Dataset
        objects and information about each associated file (e.g., filename, text/binary flag, etc.).  Upon return, the
        'GetDatasetWriteTicketsResponse' object will contain FMS write tickets that can be used to upload the file to
        the Teamcenter volume, and Dataset information that can be used to commit the changes to the database by using
        the 'FileManagementService' 'commitDatasetFiles'() operation.
        This operation supports the upload (addition) of files representing named references of a Dataset object to a
        Teamcenter volume.
        This operation was unintentionally published.  It is supported only for internal Siemens PLM purposes. 
        Customers should not invoke this operation.
        
        Use cases:
        This operation supports the upload (addition) of files representing named references of a Dataset object to a
        Teamcenter volume.
        """
        return cls.execute_soa_method(
            method_name='getDatasetWriteTickets',
            library='Core',
            service_date='2006_03',
            service_name='FileManagement',
            params={'inputs': inputs},
            response_cls=GetDatasetWriteTicketsResponse,
        )

    @classmethod
    def getFileReadTickets(cls, files: List[ImanFile]) -> FileTicketsResponse:
        """
        This operation obtains File Management System (FMS) read tickets for a set of supplied
        ImanFile objects.  The supplied tickets are used to transfer files from a Teamcenter volume 
        to local storage.  The 'files' input parameter contains a list of the ImanFile objects to be read 
        from the Teamcenter volume and transferred to local storage.
        FMS requires tickets for all file transfers to and from Teamcenter volumes.  An FMS read ticket is 
        required to obtain a file from a Teamcenter volume, while an FMS write ticket is needed to place a file in the
        Teamcenter volume.  It is often times more expedient to request several tickets at one time, especially if it
        is known ahead of time that many files will need to be moved.  For this reason, the caller may supply multiple
        ImanFile objects, for which FMS tickets are desired, in the input vector.
        This operation was unintentionally published.  It is supported only for internal Siemens PLM purposes. 
        Customers should not invoke this operation.
        
        Use cases:
        This operation supports the download of data files represented by ImanFile objects from a Teamcenter volume
        into a local client environment.
        """
        return cls.execute_soa_method(
            method_name='getFileReadTickets',
            library='Core',
            service_date='2006_03',
            service_name='FileManagement',
            params={'files': files},
            response_cls=FileTicketsResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Core._2010_09.FileManagement import CommitReplacedFileInfo
from tcsoa.gen.Internal.Core._2010_09.DataManagement import DatasetFileQueryInfo, DatasetFilesResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class FileManagementService(TcService):

    @classmethod
    def commitReplacedFiles(cls, commitInfos: List[CommitReplacedFileInfo], flags: List[bool]) -> ServiceData:
        """
        This operation is invoked after successfully uploading a file to a transient volume.
        These files are typically files which already exist in the Teamcenter volume.  It is not necessary that the
        files be attached to a Dataset as a named reference, though that is typical.   No Dataset or named reference
        relation is required or created during this operation.
        This operation will copy the file from its uploaded location in the transient volume into the filename
        specified in the regular Teamcenter volume.  If a new filename is specified, then the file will be moved to the
        new filename, and the previous file in the Teamcenter volume will be deleted.  If no new filename is specified,
        then the file will overwrite the original filename in the Teamcenter volume.
        This operation supports the replacement of a file in a Teamcenter volume.
        This operation is unpublished.  It is supported only for internal Siemens PLM purposes.  Customers should not
        invoke this operation.
        
        Use cases:
        This operation supports the use case of replacing files attached to an ImanFile object, in a Teamcenter
        transient volume.
        """
        return cls.execute_soa_method(
            method_name='commitReplacedFiles',
            library='Internal-Core',
            service_date='2010_09',
            service_name='FileManagement',
            params={'commitInfos': commitInfos, 'flags': flags},
            response_cls=ServiceData,
        )


class DataManagementService(TcService):

    @classmethod
    def getDatasetFiles(cls, inputs: List[DatasetFileQueryInfo], retrieveFiles: bool) -> DatasetFilesResponse:
        """
        This operation fetches the Dataset instances and files for the given query criteria.
        """
        return cls.execute_soa_method(
            method_name='getDatasetFiles',
            library='Internal-Core',
            service_date='2010_09',
            service_name='DataManagement',
            params={'inputs': inputs, 'retrieveFiles': retrieveFiles},
            response_cls=DatasetFilesResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.Manufacturing._2011_12.DataManagement import PostAssignIDICResponse, PostAssignIDICInput
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def postAssignIDICMaker(cls, postAssignInput: List[PostAssignIDICInput]) -> PostAssignIDICResponse:
        """
        This operation creates IDIC (ID in context) for assembly parts in EBOM and PMI contexts. So far, IDIC heas
        generated for occurrences in consumption level only. This operation provides functionality to generate IDIC for
        received occurrences in any level.
        """
        return cls.execute_soa_method(
            method_name='postAssignIDICMaker',
            library='Internal-Manufacturing',
            service_date='2011_12',
            service_name='DataManagement',
            params={'postAssignInput': postAssignInput},
            response_cls=PostAssignIDICResponse,
        )



from tcsoa.gen.Administration._2008_03.services import IRMService as imp0
from tcsoa.gen.Administration._2017_05.services import GroupManagementService as imp1
from tcsoa.gen.Administration._2017_05.services import RoleManagementService as imp2
from tcsoa.gen.Administration._2017_05.services import UserManagementService as imp3
from tcsoa.gen.Administration._2006_03.services import IRMService as imp4
from tcsoa.gen.Administration._2007_06.services import AuthorizationService as imp5
from tcsoa.gen.Administration._2008_12.services import AuthorizationService as imp6
from tcsoa.gen.Administration._2007_01.services import UserManagementService as imp7
from tcsoa.gen.Administration._2015_07.services import UserManagementService as imp8
from tcsoa.gen.Administration._2012_09.services import PreferenceManagementService as imp9
from tcsoa.gen.Administration._2016_03.services import UserManagementService as imp10
from tcsoa.gen.Administration._2012_10.services import IRMService as imp11
from tcsoa.gen.Administration._2010_04.services import IRMService as imp12
from tcsoa.gen.Administration._2016_10.services import UserManagementService as imp13
from tcsoa.gen.Administration._2010_04.services import DisciplineManagementService as imp14
from tcsoa.gen.Administration._2018_11.services import IRMService as imp15
from tcsoa.gen.Administration._2018_11.services import OrganizationManagementService as imp16
from tcsoa.gen.Administration._2012_09.services import UserManagementService as imp17
from tcsoa.gen.Administration._2008_06.services import PreferenceManagementService as imp18
from tcsoa.gen.Administration._2014_10.services import UserManagementService as imp19
from tcsoa.gen.Administration._2011_05.services import PreferenceManagementService as imp20
from tcsoa.gen.Administration._2007_06.services import PreferenceManagementService as imp21
from tcsoa.gen.Administration._2015_03.services import UserManagementService as imp22
from tcsoa.utils import TcService


class IRMService(TcService):
    activateUsers = imp0.activateUsers
    checkAccessorsPrivileges = imp4.checkAccessorsPrivileges
    deactivateUsers = imp0.deactivateUsers
    getAMImpactedObjects = imp11.getAMImpactedObjects
    getAccessorTypes = imp12.getAccessorTypes
    getEffectiveACLInfo = imp4.getEffectiveACLInfo
    getEffectiveACLInfo2 = imp12.getEffectiveACLInfo2
    getExtraProtectionInfo = imp4.getExtraProtectionInfo
    getExtraProtectionInfo2 = imp12.getExtraProtectionInfo2
    getPrivilegeNames = imp12.getPrivilegeNames
    getSessionInfoFromTicket = imp15.getSessionInfoFromTicket
    getSessionInfoTicket = imp15.getSessionInfoTicket
    getSessionValues = imp11.getSessionValues
    removeAccessor = imp4.removeAccessor
    setPrivileges = imp4.setPrivileges


class GroupManagementService(TcService):
    addChildGroups = imp1.addChildGroups


class RoleManagementService(TcService):
    addRolesToGroup = imp2.addRolesToGroup
    removeRolesFromGroup = imp2.removeRolesFromGroup


class UserManagementService(TcService):
    addUsersAsGroupMembers = imp3.addUsersAsGroupMembers
    createDisciplines = imp7.createDisciplines
    createOrUpdateUser = imp8.createOrUpdateUser
    deleteUser = imp10.deleteUser
    getCurrentCountryPageInfo = imp13.getCurrentCountryPageInfo
    getUserGroupMembers = imp17.getUserGroupMembers
    makeUser = imp19.makeUser
    removeGroupMembers = imp3.removeGroupMembers
    saveAndValidateCurrentCountry = imp13.saveAndValidateCurrentCountry
    setGroupMemberProperties = imp17.setGroupMemberProperties
    setUserProfileProperties = imp22.setUserProfileProperties


class AuthorizationService(TcService):
    checkAuthorization = imp5.checkAuthorization
    checkAuthorizationAccess = imp6.checkAuthorizationAccess


class PreferenceManagementService(TcService):
    deletePreferenceDefinitions = imp9.deletePreferenceDefinitions
    deletePreferencesAtLocations = imp9.deletePreferencesAtLocations
    getPreferences = imp9.getPreferences
    getPreferencesAtLocations = imp9.getPreferencesAtLocations
    importPreferencesAtLocationDryRun = imp9.importPreferencesAtLocationDryRun
    importPreferencesAtLocations = imp9.importPreferencesAtLocations
    lockSitePreferences = imp18.lockSitePreferences
    refreshPreferences = imp20.refreshPreferences
    removeStalePreferenceInstancesAtLocations = imp9.removeStalePreferenceInstancesAtLocations
    setPreferences = imp21.setPreferences
    setPreferences2 = imp9.setPreferences2
    setPreferencesAtLocations = imp9.setPreferencesAtLocations
    setPreferencesDefinition = imp9.setPreferencesDefinition
    unlockSitePreferences = imp18.unlockSitePreferences


class DisciplineManagementService(TcService):
    getDiscipline = imp14.getDiscipline


class OrganizationManagementService(TcService):
    getUserConsentStatement = imp16.getUserConsentStatement
    recordUserConsent = imp16.recordUserConsent



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ConnectResponse(TcBaseObj):
    """
    Holds a ServiceData (ConnectResponse.serviceData), and number of licenses avaliable.
    
    :var outputVal: outputVal
    :var serviceData: serviceData
    """
    outputVal: int = 0
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Classification._2007_01.Classification import ClassDef
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetLibraryHierarchyResponse(TcBaseObj):
    """
    Holds classification objects returned by 'getLibraryHierarchy()'' 'method.
    
    :var clsInfo: List of class references found for this library
    :var svcData: Any failures with Key-LOV ID mapped to the error message in the 'ServiceData' list of partial errors.
    """
    clsInfo: List[ClassInfo] = ()
    svcData: ServiceData = None


@dataclass
class HierarchyInfoAndOptions(TcBaseObj):
    """
    Structure representing the ClassId and options
    
    :var id: Class ID to be deleted.
    :var theRecurseOption: Flag to indicate if the delete operation should be recursively executed on the child classes
    for the given class.
    :var theViewsOption: Flag to indicate if the Views associated with the class being deleted should be removed first.
    If views exist and this flag does not indicate deleting them first, then a referential integrity error will be
    generated for the given class.
    :var theIcosOption: Flag to indicate if the Classification objects (ICO) should be delete first. If ICOs exist and
    this flag does not indicate deleting them first, then a referential integrity error will be generated for the given
    class.
    :var theWSOOption: Flag to indicate if the classified Workspace Objects should be deleted first.
    :var theChildrenOnlyOption: Flag to indicate if only the child classes should be deleted.
    :var theIgnoreOption: Flag to indicate if the operation to should continue on error.
    - FALSE: stop on first error 
    - TRUE: continue on error and report all failed objects back in the list of failed objects.
    
    """
    id: str = ''
    theRecurseOption: bool = False
    theViewsOption: bool = False
    theIcosOption: bool = False
    theWSOOption: bool = False
    theChildrenOnlyOption: bool = False
    theIgnoreOption: bool = False


@dataclass
class ClassInfo(TcBaseObj):
    """
    Structure representing class description in ClassDef structure and business object
    
    :var classDefn: Reference of the class definition structures holding all the class properties for the classes
    retrieved by this operation
    :var classBO: Reference to the business object found associated to this class.
    """
    classDefn: ClassDef = None
    classBO: BusinessObject = None



from __future__ import annotations

from tcsoa.gen.Classification._2009_10.Classification import AutoComputeAttributesMap, GetAttributesForClassesResponse2, AutoComputeAttributesResponse, GetKeyLOVsResponse2, KeyLOVDefinition2
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService
from tcsoa.gen.BusinessObjects import WorkspaceObject


class ClassificationService(TcService):

    @classmethod
    def createOrUpdateKeyLOVs(cls, keyLOVsInput: List[KeyLOVDefinition2]) -> ServiceData:
        """
        The operation creates or updates  the key-LOV objects based on the input such as name, id etc., if the input ID
        matches that of an existing key-LOV, it will be updated. Else new key-LOV object will be created. A key-LOV is
        a list of values used in classification. The key-LOVs are used to define one or more values that can be set for
        classification dictionary attributes
        
        Typical format of a Key-LOV 
        
            <key-LOV ID>:<key-LOV Name>
            <Key10>:<Value10>
            <Key20>:<Value20>
        
        Example of a Key-LOV:
        
        -33381 : Design Categories
            Des1 : Bearing
            Des2 : Bracket
            Des3 : Frame
            Des4 : LeadBox
        
        
        Use cases:
        User wants to create new key-LOVs to be used with classification or need to update the existing ones in
        classification.
        
        Exceptions:
        >Throws 'ServiceException' (SOA Framework class that holds model objects and partial errors) when
        classification system fails to create or update Key-LOV values for the given Key-LOV ID
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateKeyLOVs',
            library='Classification',
            service_date='2009_10',
            service_name='Classification',
            params={'keyLOVsInput': keyLOVsInput},
            response_cls=ServiceData,
        )

    @classmethod
    def autoComputeAttributes(cls, icoId: str, wso: WorkspaceObject, classId: str, viewId: str, inputAttrs: AutoComputeAttributesMap, unitSystem: int, mode: int) -> AutoComputeAttributesResponse:
        """
        Computes the attribute values  of classification object based on other attribute values within the same object
        or an associated classification view. Or the value can be computed based on attribute values of the object
        being classified. A classification object is also called ICO.
        
        Use cases:
        User need to automatically compute classification attribute values for attributes marked as 'AutoComputed'. The
        values can be computed based on - other attribute values belonging to same classification object or an
        associated classification view or attribute values of the object being classified.
        
        Exceptions:
        >The operation will only throw a 'ServiceException' in cases when an error condition cannot be handled and an
        appropriate error message could not be added to the list of partial errors in 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='autoComputeAttributes',
            library='Classification',
            service_date='2009_10',
            service_name='Classification',
            params={'icoId': icoId, 'wso': wso, 'classId': classId, 'viewId': viewId, 'inputAttrs': inputAttrs, 'unitSystem': unitSystem, 'mode': mode},
            response_cls=AutoComputeAttributesResponse,
        )

    @classmethod
    def getAttributesForClasses2(cls, classIds: List[str]) -> GetAttributesForClassesResponse2:
        """
        Provides information on class attributes for the classification classes based on input classification class
        ids. Detailed information about class attributes is provided & includes class attribute name, description,
        format, unit system, minimum/maximum value, configuration set & extended properties.
        
        Use cases:
        When user wants to view details of all class attributes associated with a classification class. The method is
        similar to 'getAttributesForClasses()' method, but provides information in a slightly different format. Also
        additional information like that on the extended properties of class attributes is provided
        
        Exceptions:
        >The operation will only throw a 'ServiceException' for an unknown type of exception.
        In all other cases failures will be returned with the Class ID mapped to the error message in the 'ServiceData'
        list of partial errors of the returned 'GetAttributesForClassesResponse2' structure.
        """
        return cls.execute_soa_method(
            method_name='getAttributesForClasses2',
            library='Classification',
            service_date='2009_10',
            service_name='Classification',
            params={'classIds': classIds},
            response_cls=GetAttributesForClassesResponse2,
        )

    @classmethod
    def getKeyLOVs2(cls, keyLOVIds: List[str]) -> GetKeyLOVsResponse2:
        """
        Gets the information for classification key-LOVs  based on given ID(s). Information such as key-LOV's name,
        display options, owning site, shared sites, deprecation status and key and value entries can be obtained. A
        key-LOV is a list of values used in classification. The key-LOVs are used to define one or more values that can
        be set for classification dictionary attributes
        
        Typical format of a Key-LOV -
        
            <key-LOV ID>:<key-LOV Name>
            <Key10>:<Value10>
            <Key20>:<Value20>
         
        Example of a KeyLOV:
        
        - 33381:Design Categories
            Des1:Bearing
            Des2:Bracket
            Des3:Frame
            Des4:LeadBox
        
        Use cases:
        User wants to retrieve the information for an existing key-LOV using the key-LOV's unique identifier. This
        operation is similar to 'getKeyLOVs''()'operation, but provides more detailed information about the required
        key-LOV.
        
        Exceptions:
        >The operation will only throw a 'ServiceException' for an unknown type of exception.In all other cases
        failures will be returned with the Key-LOV ID mapped to the error message in the 'ServiceData' list of partial
        errors of 'GetKeyLOVsResponse' return structure.
        """
        return cls.execute_soa_method(
            method_name='getKeyLOVs2',
            library='Classification',
            service_date='2009_10',
            service_name='Classification',
            params={'keyLOVIds': keyLOVIds},
            response_cls=GetKeyLOVsResponse2,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Group, Role, User
from tcsoa.gen.Core._2009_04.ProjectLevelSecurity import LoadProjectDataForUserResponse
from tcsoa.utils import TcService


class ProjectLevelSecurityService(TcService):

    @classmethod
    def getDefaultProject(cls, tcUser: User, tcGroup: Group, tcRole: Role) -> LoadProjectDataForUserResponse:
        """
        This operation returns a list of the default projects for a given user, group and role combination. If no user
        is specified, it looks for the default project of the current logged in user. If no group and role is
        specified, all default projects for the specified user  are returned. When TC_show_all_user_projects is set to
        TRUE, projects are reteturned regardless of the given user&rsquo;s group and role. Therefore the default
        projects are returned without checking if the user is a member of the default project in the specified group
        and role. When TC_show_all_user_projects is set to FALSE, projects are returned only when the given user has
        the membership in the specified group and role. Therefore the default projects are returned as empty
        
        Use cases:
        Use Case 1: Changing group/role selection in the user setting dialog. The default project will be automatically
        selected in the project field, if the user has the membership in the project with given group and role
        """
        return cls.execute_soa_method(
            method_name='getDefaultProject',
            library='Core',
            service_date='2016_10',
            service_name='ProjectLevelSecurity',
            params={'tcUser': tcUser, 'tcGroup': tcGroup, 'tcRole': tcRole},
            response_cls=LoadProjectDataForUserResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Dataset, ImanFile
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetPlmdFileTicketResponse(TcBaseObj):
    """
    Return structure for the getPlmdFileTicketForUpload and getPlmdFileTicketForDownload operations.
    
    :var ticket: The requested plmd file transient ticket.
    :var serviceData: Holds any partial errors that may have occurred during the operation.
    """
    ticket: str = ''
    serviceData: ServiceData = None


@dataclass
class DatashareManagerDownloadInfo(TcBaseObj):
    """
    Information about a file to be downloaded asynchronously via Datashare Manager 
    
    :var dataset: The Dataset object from which the ImanFile object(s) representing the data files that need to be
    downloaded from the Teamcenter volume.
    :var imanFile: A ImanFile object that need to be downloaded from a Teamcenter volume.
    :var absoluteFilePath: The absolute file path of the file being downloaded on the client host.The value can be set
    to an empty  string("") to download the file to the default staging directory on the client host.
    """
    dataset: Dataset = None
    imanFile: ImanFile = None
    absoluteFilePath: str = ''


@dataclass
class DatashareManagerUploadInfo(TcBaseObj):
    """
    Information about a file to be uploaded asynchronously via Datashare Manager.
    
    :var dataset: The Dataset object from which the ImanFile object(s) representing the data files that need to be
    uploaded from the Teamcenter volume.
    :var namedReferenceName: The named reference realtion to the ImanFile object file.
    :var absoluteFilePath: The absolute file path of the file being uploaded on the client host
    """
    dataset: Dataset = None
    namedReferenceName: str = ''
    absoluteFilePath: str = ''



from tcsoa.gen.Internal_Validation._2007_06.services import ValidationService as imp0
from tcsoa.gen.Internal_Validation._2012_02.services import ValidationService as imp1
from tcsoa.gen.Internal_Validation._2008_06.services import ValidationService as imp2
from tcsoa.gen.Internal_Validation._2010_04.services import ValidationService as imp3
from tcsoa.utils import TcService


class ValidationService(TcService):
    createOrUpdateValidationAgents = imp0.createOrUpdateValidationAgents
    createOrUpdateValidationData = imp0.createOrUpdateValidationData
    createOrUpdateValidationResults = imp0.createOrUpdateValidationResults
    createOrUpdateValidationResults2 = imp1.createOrUpdateValidationResults2
    evaluateValidationResultsWithRules = imp2.evaluateValidationResultsWithRules
    getValidationResults = imp0.getValidationResults
    getValidationResults2 = imp3.getValidationResults2
    getValidationTargets = imp3.getValidationTargets
    parseRequirementExpressions = imp2.parseRequirementExpressions
    performActionOnOverrideApproval = imp1.performActionOnOverrideApproval
    runValidation = imp3.runValidation



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.AWS2._2016_03.Finder import ColumnConfigInput, SearchInput, SaveColumnConfigData
from tcsoa.gen.Internal.AWS2._2017_12.DataManagement import GetViewModelProps2Response, GetTableViewModelPropertiesIn, GetAvailableWorkspacesResponse, TCSessionAndAnalyticsInfo, LoadViewModelForEditing2Response, StringMap8, GetTableViewModelPropsResp, FilterParams
from tcsoa.gen.Internal.AWS2._2017_12.FullTextSearch import GetIndexedObjectsAndUpdateResponse, QueryAndUpdateSyncDataResponse, QueryAndUpdateDataInput
from typing import List
from tcsoa.gen.Internal.AWS2._2017_06.DataManagement import LoadViewModelForEditingInfo
from tcsoa.gen.Internal.AWS2._2017_12.Finder import SearchResponse4
from tcsoa.utils import TcService
from datetime import datetime


class DataManagementService(TcService):

    @classmethod
    def getTCSessionAnalyticsInfo(cls, extraInfo: List[str]) -> TCSessionAndAnalyticsInfo:
        """
        This operation provides information about the current user's Teamcenter session and data required for
        Teamcenter Software Analytics processing.
        
        Use cases:
        Use Case 1: After the user logs into Active Workspace this operation is invoked by the Active Workspace client
        to get information related to the user session like Groups, Projects, the Teamcenter Server version, platform
        type,License type, preference related to the time interval and product excellence agreement as required by
        analytics.
        """
        return cls.execute_soa_method(
            method_name='getTCSessionAnalyticsInfo',
            library='Internal-AWS2',
            service_date='2017_12',
            service_name='DataManagement',
            params={'extraInfo': extraInfo},
            response_cls=TCSessionAndAnalyticsInfo,
        )

    @classmethod
    def getTableViewModelProperties(cls, input: GetTableViewModelPropertiesIn) -> GetTableViewModelPropsResp:
        """
        The operation returns the column configuration and Table View Model properties for the input object UIDs.
        
        Use cases:
        Usecase 1: When the user opens a structure in the content tab and switch to table mode. The column
        configuration and related properties need to be fetched.
        
        Usecase 2: When a structure is opened in the tree mode. The column configuration and related properties need to
        be fetched.
        """
        return cls.execute_soa_method(
            method_name='getTableViewModelProperties',
            library='Internal-AWS2',
            service_date='2017_12',
            service_name='DataManagement',
            params={'input': input},
            response_cls=GetTableViewModelPropsResp,
        )

    @classmethod
    def getViewModelProperties2(cls, objects: List[BusinessObject], attributes: List[str], configs: StringMap8) -> GetViewModelProps2Response:
        """
        This service operation is provided to get property values of instances outside of the current object property
        policy for a particular business object. This operation is an enhanced version of the getProperties service
        operation and allows for retrieving values of dynamic compound properties as well.
        
        Use cases:
        Use Case 1: Open an object in ActiveWorkspace.
        When an object is selected in ActiveWorkspace, the user may choose to invoke the open operation. When that
        operation is executed the getViewModelProperties call is executed to fetch required properties.
        """
        return cls.execute_soa_method(
            method_name='getViewModelProperties2',
            library='Internal-AWS2',
            service_date='2017_12',
            service_name='DataManagement',
            params={'objects': objects, 'attributes': attributes, 'configs': configs},
            response_cls=GetViewModelProps2Response,
        )

    @classmethod
    def loadViewModelForEditing2(cls, inputs: List[LoadViewModelForEditingInfo]) -> LoadViewModelForEditing2Response:
        """
        This method ensures that the properties can be edited and returns the last save date of the related objects for
        optimistic edit. This new operation is similar to the existing loadDataForEditing operation, the difference is
        that this SOA saves the ViewModelObject instead of model object, and this SOA supports dynamic compound
        properties. In addition to checking if the object and the properties are modifiable, if the property is dynamic
        compound property, this SOA also validates if the traversal path of the dynamic compound property is same as
        when properties was queries from server during display, if it is, then editing is not allowed.
        loadViewModelForEditing operation will be deprecated.
        """
        return cls.execute_soa_method(
            method_name='loadViewModelForEditing2',
            library='Internal-AWS2',
            service_date='2017_12',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=LoadViewModelForEditing2Response,
        )

    @classmethod
    def getAvailableWorkspaces(cls, filterParams: FilterParams) -> GetAvailableWorkspacesResponse:
        """
        This operation returns available workspaces. The list of workspaces is populated based on input criteria (e.g.
        current user session ).
        
        Use cases:
        Use Case 1: Click on the current workspace user is working in. 
        In Active Workspace, when user clicks on the header context link (which shows current workspace) the
        getAvailableWorkspaces operation will be called. The client will show a list of available workspaces to the
        user to choose a value from. 
        
        Use Case 2: Show list of all workspaces in primary workarea.
        In ActiveWorkspace, when user navigates to "workspaces" sublocation, primary work-area needs to display all the
        workspaces in the system. In order to populate the primary workarea, the getAvailableWorkspaces call is
        invoked. The returned data is parsed and processed in order to display list of all workspaces into the system.
        """
        return cls.execute_soa_method(
            method_name='getAvailableWorkspaces',
            library='Internal-AWS2',
            service_date='2017_12',
            service_name='DataManagement',
            params={'filterParams': filterParams},
            response_cls=GetAvailableWorkspacesResponse,
        )


class FinderService(TcService):

    @classmethod
    def performSearchViewModel2(cls, searchInput: SearchInput, columnConfigInput: ColumnConfigInput, saveColumnConfigData: SaveColumnConfigData, inflateProperties: bool) -> SearchResponse4:
        """
        This operation routes a search request to a specific provider specified as providerName in the searchInput
        structures.
        
        The framework allows a custom solution to be able to provide a new specific provider to collect data, perform
        sorting and filtering. The provider can be a User Inbox retriever or Full Text searcher, for example. The new
        data provider can be encapsulated via a new runtime business object from Fnd0BaseProvider class. The
        implementation is done using its fnd0performSearch operation.
        
        RuntimeBusinessObject 
        ---- Fnd0BaseProvider (foundation template) 
        -------- Fnd0GetChildrenProvider(foundation template) 
        -------- Awp0FullTextSearchProvider (aws template)
        -------- Awp0TaskSearchProvider (aws template) 
        -------- Aos0TestProvider (aosinternal template) 
        -------- etc.
        
        This operation provides a common interface to send the request to and receive the response from a new data
        provider. Ultimately it allows common framework in UI to support filter, pagination, and sorting. 
        
        This operation allows the caller to send the search input, filter values, and sorting data. These input values
        are then to be passed to the fnd0performSearch operation on the server to collect, sort, and filter its
        results. 
        
        The first two input parameters are important. The first input is the provider name. This is a string to
        represent the type name of RuntimeBusinessObject which this request should be routed to. If the template that
        contains the class is not installed, a partial error 217016 is returned. The second input is the search input
        map. The key is different per each provider. For example, for Full Text searcher, the input key would be a
        searchString. For User Inbox retriever, the input key would be an Inbox Type. The fnd0performSearch
        implementation for each provider takes into account the key name as it is used to store the values in
        OperationInput object. 
        
        The internal property name is passed as part of search input to allow grouping of business objects. The
        grouping allows for added functionality such as color coding, etc. If no internal property name is passed,
        default value is assumed as "object_type" to group the business objects. 
        
        If new list of column config data is passed in to be saved, it will get saved in the database for the login
        user scope. This will be done prior to performing the search. 
        
        The column configuration input has the criteria needed to query for the columns configured for the provided
        client and client scope URI. The final list of columns are identified based on the types returned back in the
        search results and the operation type (Union / Intersection / Configured ) configured for the client scope URI.
        Finally the determined column attributes and the attributes to be inflated passed in as part of search input
        criteria are combined. This combined list of attributes along with the business objects found during search are
        added to the Service data.
        
        Use cases:
        Use Case 1: User clicks on Inbox tile
        A search request will be made by passing in the needed criteria. In this case the provider name is
        "Awp0InboxProvider". It queries for the data and finds the effective columns for the client scope URI passed in
        through column config input. The results are finally shown in the view type selected in client. If table view
        is selected the returned effective columns will be shown.
        
        Use Case 2: User performs search by entering a keyword in global search input box. 
        A search request will be made by passing in the needed criteria that includes the keyword user has typed in. In
        this case the provider name is "Awp0FullTextSearchProvider". It queries for the data and finds the effective
        columns for the client scope URI passed in through column config input. The results are finally shown in the
        view type selected in client. If table view is selected the returned effective columns will be shown. 
        
        Use Case 3: User tries to save new column configurations after performing search. 
        The new list of column configurations will be saved for the login user scope. Later the search request will be
        made by passing in the needed criteria that includes the keyword user had typed in. In this case the provider
        name is "Awp0FullTextSearchProvider". It queries for the data and finds the effective columns for the client
        scope URI passed in through column config input. The results are finally shown in the view type selected in
        client. If table view is selected the returned effective columns will be shown.
        
        ---
        """
        return cls.execute_soa_method(
            method_name='performSearchViewModel2',
            library='Internal-AWS2',
            service_date='2017_12',
            service_name='Finder',
            params={'searchInput': searchInput, 'columnConfigInput': columnConfigInput, 'saveColumnConfigData': saveColumnConfigData, 'inflateProperties': inflateProperties},
            response_cls=SearchResponse4,
        )


class FullTextSearchService(TcService):

    @classmethod
    def queryAndUpdateSyncData(cls, inputData: QueryAndUpdateDataInput) -> QueryAndUpdateSyncDataResponse:
        """
        This service operation queries for the data to be synced and updates the state of those records in ACCT_TABLE. 
        The Full Text Search Indexer which indexes Teamcenter data uses the information for further processing.
        
        Queries for deleted objects that have been previously indexed after previous sync.
        Updates these records in ACCT_TABLE with DELETE_PENDING state
        Updates the :FTS_DEL_QRY app_id timetsamp in SUBSCRIPTION_TABLE
        
        Queries for newly added objects after previous sync.
        Inserts these records into ACCT_TABLE with ADD_MODIFY_PENDING state.
        Updates the timestamp of the :FTS_ADD_QRY app_id in SUBSCRIPTION_TABLE
        
        Queries for modifications via relations of indexed objects
        Updates these records in ACCT_TABLE with MODIFY_REL_PENDING state.
        Queries for modifications of indexed objects.
        Updates these records in ACCT_TABLE with ADD_MODIFY_PENDING state
        Updates the timestamp of the :FTS_SYNC app_id in SUBSCRIPTION_TABLE
        
        The count of added or deleted or modified objects found by queries will be returned back through SyncInfo map.
        Any errors during queries or update of records will be reported as partial errors through ServiceData object.
        """
        return cls.execute_soa_method(
            method_name='queryAndUpdateSyncData',
            library='Internal-AWS2',
            service_date='2017_12',
            service_name='FullTextSearch',
            params={'inputData': inputData},
            response_cls=QueryAndUpdateSyncDataResponse,
        )

    @classmethod
    def getIndexedObjectsAndUpdate(cls, applicationID: str, subscriptionAppID: str, exportedDate: datetime, statuses: List[int], classNames: List[str], maxObjectCount: int, statusToUpdate: int) -> GetIndexedObjectsAndUpdateResponse:
        """
        This operation queries for the indexed objects matching the given input criteria. Criteria include application
        ID, timestamp, list of export or import statuses of the object and list of object class names. The
        maxCountObject can be specified to limit the count of matching object UIDs to be fetched by the query.  The
        statusToUpdate will be used to update the records of the UIDs fetched by the query in ACCT_TABLE.
        """
        return cls.execute_soa_method(
            method_name='getIndexedObjectsAndUpdate',
            library='Internal-AWS2',
            service_date='2017_12',
            service_name='FullTextSearch',
            params={'applicationID': applicationID, 'subscriptionAppID': subscriptionAppID, 'exportedDate': exportedDate, 'statuses': statuses, 'classNames': classNames, 'maxObjectCount': maxObjectCount, 'statusToUpdate': statusToUpdate},
            response_cls=GetIndexedObjectsAndUpdateResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Item, NamedVariantExpression, BOMLine, MEAppearancePathNode
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetProductItemResponse(TcBaseObj):
    """
    The 'GetProductItemResponse' structure contains a list of all Product Items found in the database and a
    'ServiceData' object.
    
    :var output: A list of  all Product Items found in the database.
    :var serviceData: The 'ServiceData' object containing error codes and error messages in case of failure.
    """
    output: List[Item] = ()
    serviceData: ServiceData = None


@dataclass
class GetRemoveABEPartsResponse(TcBaseObj):
    """
    The 'GetRemoveABEPartsResponse' structure contains the 'ServiceData' object.
    
    :var serviceData: The 'ServiceData' containing error codes and error messages along with the indices for which the
    'removePartsRelatedToABE' operation fails.
    """
    serviceData: ServiceData = None


@dataclass
class OccNotesAttributes(TcBaseObj):
    """
    The part data information about each component is provided
    by way of the OccNotesAttributes data structure.
    
    :var noteType: Note type for selected component
    :var noteText: Note text for selected component
    """
    noteType: str = ''
    noteText: str = ''


@dataclass
class RemoveABEPartsInputInfo(TcBaseObj):
    """
    The information to remove parts related to ABE is provided by way of the 'RemoveABEPartsInputInfo' data structure.
    
    :var abeAPN: The reference to the Appearance Path Node of the Architecture Breakdown Element of which Part
    Solutions are to be removed. This is a mandatory attribute for the operation to succeed and cannot be null.
    :var topline: The reference to the topline to which Architecture Breakdown Element is linked. This is a mandatory
    attribute for the operation to succeed and cannot be null.
    """
    abeAPN: MEAppearancePathNode = None
    topline: BOMLine = None


@dataclass
class ReplacePartSolutionInputInfo(TcBaseObj):
    """
    The information to replace part is provided by way of the
    ReplacePartSolutionInputInfo data structure.
    
    :var values: Vector of OccNotesAttributes for each added component
    :var component: Component by which the replacement has been doing
    :var bomLine: Bomline to which the part is to be replaced
    :var abeAPN: APN of the associated ABE
    :var aNves: Vector of authorized NVEs present on ABE
    :var splitAndClone: splitAndClone
    :var carrySubstitutes: carrySubstitutes
    """
    values: List[OccNotesAttributes] = ()
    component: Item = None
    bomLine: BOMLine = None
    abeAPN: MEAppearancePathNode = None
    aNves: List[NamedVariantExpression] = ()
    splitAndClone: bool = False
    carrySubstitutes: bool = False


@dataclass
class ReplacePartSolutionResponse(TcBaseObj):
    """
    The ReplacePartSolutionResponse structure represents one output
    vector ReplacePartSolutionOutputInfo and the service data.
    
    :var component: component
    :var serviceData: serviceData returned as response for retrieving
    replace part in product information
    """
    component: List[Item] = ()
    serviceData: ServiceData = None


@dataclass
class AddPartSolutionInputInfo(TcBaseObj):
    """
    The information of added part is provided by way of the
    AddPartSolutionInputInfo data structure.
    
    :var values: Vector of OccNotesAttributes for each added component
    :var component: Component to be added
    :var abe: BomLine to which the component is to be added
    :var prodRevTags: Vector of product revisions for each selected component
    :var abeApnTag: Appearance Path Node for each added component
    :var aNves: Vector of authorized NVEs present on ABE
    :var quantity: Quantity of part to be added
    """
    values: List[OccNotesAttributes] = ()
    component: Item = None
    abe: BOMLine = None
    prodRevTags: List[Item] = ()
    abeApnTag: MEAppearancePathNode = None
    aNves: List[NamedVariantExpression] = ()
    quantity: int = 0


@dataclass
class AddPartSolutionResponse(TcBaseObj):
    """
    The AddPartSolutionResponse structure represents one output
    vector AddPartSolutionOutputInfo and the service data.
    
    :var partSolutionAPNs: partSolutionAPNs
    :var serviceData: serviceData returned as response
    for retrieving add part to product information
    """
    partSolutionAPNs: List[MEAppearancePathNode] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Internal.Core._2018_06.LogicalObject import MemberPropertyDefinition2
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class PresentedPropertyDefinition2(TcBaseObj):
    """
    The presented property definitions which contains the root or member object name, source property name and an
    indicator to identify if the presented property is writable.
    
    :var presentedPropertyName: Name of the presented property.
    :var displayName: Display name of the presented property.
    :var description: Description of the presented property.
    :var rootOrMemberName: Root business object name if the presented property is from the root on the Logical Object
    type.  Member property name if the presented property is from the member on the Logical Object type.
    :var sourcePropertyName: Source property name.
    :var isWritable: If true, the presented property is writable
    """
    presentedPropertyName: str = ''
    displayName: str = ''
    description: str = ''
    rootOrMemberName: str = ''
    sourcePropertyName: str = ''
    isWritable: bool = False


@dataclass
class AddMemAndPresentedPropsWriteInput(TcBaseObj):
    """
    Represents the definition data for member and presented properties to be added to an existing logical object type.
    
    :var logicalObjectType: An object of logical object type (Fnd0LogicalObject) to which members and presented
    properties are to be added.
    :var memberPropertiesDefinitions: A list of member defintions.
    :var presentedPropertiesDefinitions: A list of presented property defintions.
    """
    logicalObjectType: BusinessObject = None
    memberPropertiesDefinitions: List[MemberPropertyDefinition2] = ()
    presentedPropertiesDefinitions: List[PresentedPropertyDefinition2] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine
from tcsoa.gen.StructureManagement._2007_06.PublishByLink import CompletenessCheckResponse, SourceAndTargets, GetSourceTopLevelResponse, LineAndWindow, PublishLinkInfo, PublishDataInfo, LogicallyEquivalentLinesResponse, PublishLinksResponse, FindTargetsResponse, FindSourceResponse, CompletenessCheckInputData
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class PublishByLinkService(TcService):

    @classmethod
    def getSourceTopLevel(cls, targets: List[BOMLine]) -> GetSourceTopLevelResponse:
        """
        Finds source of PublishLink for given target BOMLine. For the source of the PublishLink finds context PSBOMView.
        
        Use cases:
        Find context PSBOMView for the source of PublishLink.
        """
        return cls.execute_soa_method(
            method_name='getSourceTopLevel',
            library='StructureManagement',
            service_date='2007_06',
            service_name='PublishByLink',
            params={'targets': targets},
            response_cls=GetSourceTopLevelResponse,
        )

    @classmethod
    def completenessCheck(cls, input: List[CompletenessCheckInputData]) -> CompletenessCheckResponse:
        """
        Recursively evaluates completeness for BOMLine objects having underlying component as Part.  A BOMLine which
        requires positioned designed is marked as complete if underlying PartRevision has primary representation OR
        BOMLine has source associated via PublishLink object. If a BOMLine does not need positioned designed then such
        BOMLine is marked as complete as well. This operation also supports recursively clearing completeness results.
        
        If required the BOM is expanded internally. In case of packed BOMLines, if any of the BOMLine is incomplete
        then that packed line is marked as incomplete.
        
        
        Use cases:
        - Recursively perform completeness check for Part structure.
        - Recursively clears completeness check for Part structure.
        
        """
        return cls.execute_soa_method(
            method_name='completenessCheck',
            library='StructureManagement',
            service_date='2007_06',
            service_name='PublishByLink',
            params={'input': input},
            response_cls=CompletenessCheckResponse,
        )

    @classmethod
    def publishData(cls, publishDataInfos: List[PublishDataInfo]) -> PublishLinksResponse:
        """
        Copies 'Absolute Transformation Matrix' and/or JT from source BOMLine to target BOMLine objects. If a
        PublishLink does not exist for source and target then a new PublishLink is created with input dataFlags. In
        case if PublishLink already exists then dataFlags of the PublishLink object is updated.
        
        Input 'dataFlags' is used to determine which data has to be copied. Unless context was explicitly set - the
        data on target BOMLines is stored in-context of top BOMLine of the target BOMWindow. 
        
        The DirectModel Dataset is attached with Rendering relation. If the target BOMLine is an assembly then PLMXML
        file is created based on RevisionRule of the source BOMWindow and attached to target BOMLine as Text Dataset
        with Rendering relation. In case, in-context Dataset with Rendering relation already exists then that is
        replaced with the new one. The BOMWindow is saved after attaching the Dataset.
        
        Below validations are performed during operation and failures are reported in 'ServiceData'.
        
        - 'dataFlags' contains a valid value.
        - Target BOMLine requires positioned design.
        - Item type of source and target BOMLine as per PUBLISH_AlignableSourceTypes and PUBLISH_AlignableTargetTypes
        preference respectively.
        - Type name is a valid TCType name.
        - PSBOMView of the source is local.
        - No PublishLink exists other than source and target as in input.
        
        
        
        Use cases:
        Perform in-context association between occurrences of two structures and copy 'Absolute Transformation Matrix'
        and/or JT from source to target BOMLine objects.
        """
        return cls.execute_soa_method(
            method_name='publishData',
            library='StructureManagement',
            service_date='2007_06',
            service_name='PublishByLink',
            params={'publishDataInfos': publishDataInfos},
            response_cls=PublishLinksResponse,
        )

    @classmethod
    def createLinks(cls, linkInfos: List[PublishLinkInfo]) -> PublishLinksResponse:
        """
        Creates PublishLink between AbsOccurrenceViewQualifier of source and target BOMLines. The 'dataFlags' attribute
        of the PublishLink is set to 0 as no data was published. The operation also saves new PublishLink object. 
        
        Following validations are performed during operation and failures are reported in 'ServiceData'.
        
        - Item type of source and target BOMLine as per PUBLISH_AlignableSourceTypes and PUBLISH_AlignableTargetTypes
        preference respectively.
        - Type name is a valid TCType name.
        - PSBOMView of the source is local.
        - No PublishLink exists with source as input source.
        - No PublishLink exists with target as input target.
        
        
        
        Use cases:
        Perform in-context association between occurrences of two structures.
        """
        return cls.execute_soa_method(
            method_name='createLinks',
            library='StructureManagement',
            service_date='2007_06',
            service_name='PublishByLink',
            params={'linkInfos': linkInfos},
            response_cls=PublishLinksResponse,
        )

    @classmethod
    def addTargets(cls, input: List[SourceAndTargets]) -> PublishLinksResponse:
        """
        Adds AbsOccViewQualifier of target BOMLine objects to the existing PublishLink of input source BOMLine. The
        operation also saves updated PublishLink. 
        
        Following validations are performed during operation and failures are reported in 'ServiceData'.
        
        - PublishLink exists with source as input source and user has access to it.
        - Item type of source and target BOMLine as per PUBLISH_AlignableSourceTypes and PUBLISH_AlignableTargetTypes
        preference respectively.
        - No PublishLink exists with target as input target.
        - Logical identity of all target BOMLine is same.
        - Logical identity of source and target BOMLines is same.
        
        
        
        Use cases:
        Add in-context occurrence as target to an existing PublishLink for given source.
        """
        return cls.execute_soa_method(
            method_name='addTargets',
            library='StructureManagement',
            service_date='2007_06',
            service_name='PublishByLink',
            params={'input': input},
            response_cls=PublishLinksResponse,
        )

    @classmethod
    def deleteLinkForSource(cls, sources: List[BOMLine]) -> ServiceData:
        """
        Finds and deletes PublishLink for input source BOMLine objects.
        
        Use cases:
        Delete PublishLink.
        """
        return cls.execute_soa_method(
            method_name='deleteLinkForSource',
            library='StructureManagement',
            service_date='2007_06',
            service_name='PublishByLink',
            params={'sources': sources},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteTargetsFromLink(cls, targets: List[BOMLine]) -> ServiceData:
        """
        Deletes AbsOccViewQualifier of target BOMLines from the existing PublishLink.  If the target being removed is
        the last one for PublishLink then PublishLink is also deleted. Otherwise operation saves updated PublishLink.
        
        Following validations are performed during operation.
        
        PublishLink exists whose target as input
        
        
        Use cases:
        Detach target BOMLine from PublishLink.
        """
        return cls.execute_soa_method(
            method_name='deleteTargetsFromLink',
            library='StructureManagement',
            service_date='2007_06',
            service_name='PublishByLink',
            params={'targets': targets},
            response_cls=ServiceData,
        )

    @classmethod
    def findLinesWithSameLogicalIdentity(cls, input: List[LineAndWindow]) -> LogicallyEquivalentLinesResponse:
        """
        Finds logically equivalent BOMLines in BOMWindow for list of input BOMLines.
        
        BOMLines are said to be identical if their AbsoluteOccurrence objects have same 'Usage Address' and 'Positioned
        Designator'.
        
        Use cases:
        Find equivalent BOMLine objects and associate them.
        """
        return cls.execute_soa_method(
            method_name='findLinesWithSameLogicalIdentity',
            library='StructureManagement',
            service_date='2007_06',
            service_name='PublishByLink',
            params={'input': input},
            response_cls=LogicallyEquivalentLinesResponse,
        )

    @classmethod
    def findSourceInWindow(cls, input: List[LineAndWindow]) -> FindSourceResponse:
        """
        Finds source of the PublishLink in source BOMWindow for input target BOMLine objects. Source is returned as
        BOMLine.
        
        
        Use cases:
        Determine if BOMWindow has source for input target BOMLine objects.
        """
        return cls.execute_soa_method(
            method_name='findSourceInWindow',
            library='StructureManagement',
            service_date='2007_06',
            service_name='PublishByLink',
            params={'input': input},
            response_cls=FindSourceResponse,
        )

    @classmethod
    def findTargetsInWindow(cls, input: List[LineAndWindow]) -> FindTargetsResponse:
        """
        Finds target of the PublishLink in target BOMWindow for input source BOMLine objects. Targets are returned as
        BOMLine objects.
        
        Use cases:
        - Determine if BOMWindow has targets for input source BOMLine.
        - Find targets for source of PublishLink and subsequently use found target BOMLine objects to delete links.
        
        """
        return cls.execute_soa_method(
            method_name='findTargetsInWindow',
            library='StructureManagement',
            service_date='2007_06',
            service_name='PublishByLink',
            params={'input': input},
            response_cls=FindTargetsResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.ProductConfiguratorAw._2020_05.ConfiguratorManagement import GetConfiguratorDataHierarchyIn, GetConfiguratorDataHierarchyResp
from tcsoa.utils import TcService


class ConfiguratorManagementService(TcService):

    @classmethod
    def getConfiguratorDataHierarchy(cls, input: GetConfiguratorDataHierarchyIn) -> GetConfiguratorDataHierarchyResp:
        """
        This operation returns the content of Configurator Context and Dictionary based on UI widget. E.g. the data can
        be Product Line Hierarchy and / or Product Models if "Products" tab is opened and Variability data if
        "Features" tab is opened. The operation allows to sort the result by additional filters and sorting criteria
        that is provided as input.
        
        Exceptions:
        >The following service exception may be returned:
        
        77073    The operation has failed, because an invalid Configurator Perspective was passed.
        77074    The operation must contain a product item or dictionary in the Configurator Perspective.
        333012    The operation has failed because of invalid request type.
        333013    The operation has failed because of invalid parent object.
        333014    The operation has failed because of invalid Expand type.
        333015    The operation has failed because of invalid view type.
        """
        return cls.execute_soa_method(
            method_name='getConfiguratorDataHierarchy',
            library='Internal-ProductConfiguratorAw',
            service_date='2020_05',
            service_name='ConfiguratorManagement',
            params={'input': input},
            response_cls=GetConfiguratorDataHierarchyResp,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.ChangeManagement._2015_03.ChangeManagement import ConnectChangeNoticeToContextInElem, UpdateChangeNoticeRelationsIn, PreviousEffectivity, CreateOrUpdatePreviousEffResp, UpdateChangeNoticeRelationsResp, ConnectChangeNoticeToContextResp
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ChangeManagementService(TcService):

    @classmethod
    def disconnectChangeNoticeFromContext(cls, input: List[BusinessObject]) -> ServiceData:
        """
        This operation disconnects a previously associated ChangeNoticeRevision from the context (currently, the
        context must be a BOMWindow). The ChangeNoticeRevision would have been associated to a BOMWindow using the
        runtime property "cm0ChangeItemRev" or the connectChangeNoticeToContext operation. If no ChangeNoticeRevision
        is currently associated with the BOMWindow, the operation does not do any further processing for that
        particular element in the input.
        """
        return cls.execute_soa_method(
            method_name='disconnectChangeNoticeFromContext',
            library='Internal-ChangeManagement',
            service_date='2015_03',
            service_name='ChangeManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def removePrevEffectivityFromChgNotice(cls, input: List[BusinessObject]) -> ServiceData:
        """
        This operation disconnects a previously associated ConfigurationContext from ChangeNoticeRevision. If previous
        effectivity does not exist on the ChangeNoticeRevision, the operation proceeds to the next input element.
        """
        return cls.execute_soa_method(
            method_name='removePrevEffectivityFromChgNotice',
            library='Internal-ChangeManagement',
            service_date='2015_03',
            service_name='ChangeManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def updateChangeNoticeRelations(cls, inputs: List[UpdateChangeNoticeRelationsIn]) -> UpdateChangeNoticeRelationsResp:
        """
        This operation manages the secondary ItemRevision objects related to the ChangeNoticeRevision associated with a
        BOMWindow, by adding/removing these objects to/from the relations. The managed secondary ItemRevision objects
        will have appropriate sharing of the release statuses of the ChangeNoticeRevision, based on the sharing mode
        controlled by BOMWindow property cm0ChangeNoticeRevShareMode. If this property is not set, the default "Share"
        mode will be used.
        """
        return cls.execute_soa_method(
            method_name='updateChangeNoticeRelations',
            library='Internal-ChangeManagement',
            service_date='2015_03',
            service_name='ChangeManagement',
            params={'inputs': inputs},
            response_cls=UpdateChangeNoticeRelationsResp,
        )

    @classmethod
    def connectChangeNoticeToContext(cls, input: List[ConnectChangeNoticeToContextInElem]) -> ConnectChangeNoticeToContextResp:
        """
        This operation associates the given ChangeNoticeRevision to a context (currently, BOMWindow) for the purpose of
        tracking newly created revisions or revisions from revise operation in a session.
        """
        return cls.execute_soa_method(
            method_name='connectChangeNoticeToContext',
            library='Internal-ChangeManagement',
            service_date='2015_03',
            service_name='ChangeManagement',
            params={'input': input},
            response_cls=ConnectChangeNoticeToContextResp,
        )

    @classmethod
    def createOrUpdatePreviousEffectivity(cls, input: List[PreviousEffectivity]) -> CreateOrUpdatePreviousEffResp:
        """
        This operation creates a ConfigurationContext object with the provided previous effectivty parameters of
        endItem and one of unit or date. This ConfigurationContext is attached to the passed in ChangeNoticeRevision
        upon creation. If the ConfigurationContext is already present, the revision rule content is updated with the
        passed in previous effectivity parameters.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdatePreviousEffectivity',
            library='Internal-ChangeManagement',
            service_date='2015_03',
            service_name='ChangeManagement',
            params={'input': input},
            response_cls=CreateOrUpdatePreviousEffResp,
        )
